<?php

namespace Ems\Builders;

use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;
use Ems\Services\{ MinhaHistoriaService };
use Ems\Logic\ProcessamentoSolicitacao\{ ProcessamentoSolicitacaoFactory};

class SolicitacaoXlsBuilder
{
    /**
     * Gera o layout da planilha para as solicitações.
     *
     * @param array $solicitacoes Array com os dados das solicitações.
     * @return Spreadsheet O objeto Spreadsheet com os dados formatados.
     */
    public function buildSolicitacoesXls($solicitacoes): Spreadsheet
    {
        $spreadsheet = new Spreadsheet();
        $sheet = $spreadsheet->getActiveSheet();

        // Definir cabeçalhos
        $sheet->setCellValue('A1', 'Nº Vínculo');
        $sheet->setCellValue('B1', 'Vínculo');
        $sheet->setCellValue('C1', 'Estrutura Legal');
        $sheet->setCellValue('D1', 'Estrutura Gerencial');
        $sheet->setCellValue('E1', 'Nº Solicitação');
        $sheet->setCellValue('F1', 'Data de Solicitação');
        $sheet->setCellValue('G1', 'Tipo Solicitação');
        $sheet->setCellValue('H1', 'Aprovação');
        $sheet->setCellValue('I1', 'Fase Atual');
        $sheet->setCellValue('J1', 'Solicitante');
        $sheet->setCellValue('K1', 'Detalhes (Itens)');
        $sheet->setCellValue('L1', 'Fases Aprovação');

        // Ajustar largura das colunas
        $sheet->getColumnDimension('A')->setWidth(20);
        $sheet->getColumnDimension('B')->setWidth(55);
        $sheet->getColumnDimension('C')->setWidth(50);
        $sheet->getColumnDimension('D')->setWidth(50);
        $sheet->getColumnDimension('E')->setWidth(20);
        $sheet->getColumnDimension('F')->setWidth(20);
        $sheet->getColumnDimension('G')->setWidth(55);
        $sheet->getColumnDimension('H')->setWidth(20);
        $sheet->getColumnDimension('I')->setWidth(40);
        $sheet->getColumnDimension('J')->setWidth(55);
        $sheet->getColumnDimension('K')->setAutoSize(true);
        $sheet->getColumnDimension('L')->setAutoSize(true);

        // Aplicar estilo para cabeçalhos
        $headerStyle = $sheet->getStyle('A1:L1');
        $headerStyle->getFont()->setSize(12);
        $headerStyle->getFont()->setBold(true);
        $headerStyle->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);

        // Preencher os dados
        $row = 2; // Começa na segunda linha (depois dos cabeçalhos)
        foreach ($solicitacoes as $solicitacao) {
            $sheet->setCellValue('A' . $row, $solicitacao['NRVINCULOM']        ?? '');
            $sheet->setCellValue('B' . $row, $solicitacao['NMVINCULOM']        ?? '');
            $sheet->setCellValue('C' . $row, $solicitacao['NMESTRUTLEGAL']     ?? '');
            $sheet->setCellValue('D' . $row, $solicitacao['NMESTRUTGEREN']     ?? '');
            $sheet->setCellValue('E' . $row, $solicitacao['NRSOLICITACAO']     ?? '');
            $sheet->setCellValue('F' . $row, $solicitacao['DTSOLICITACAO']     ?? '');
            $sheet->setCellValue('G' . $row, $solicitacao['NMTIPOSOLICITACAO'] ?? '');
            $sheet->setCellValue('H' . $row, $solicitacao['DSAPROVACAOSEC']    ?? '');
            $sheet->setCellValue('I' . $row, $solicitacao['NMFASEAPROVSOLI']   ?? '');
            $sheet->setCellValue('J' . $row, $solicitacao['NMOPERADOR']        ?? '');
            $sheet->setCellValue('K' . $row, $solicitacao['ITEMS']             ?? '');
            $sheet->setCellValue('L' . $row, $solicitacao['FASES']             ?? '');
            $row++;
        }

        // Aplicar estilo para as células de dados
        $dataStyle = $sheet->getStyle('A2:L' . ($row - 1));
        $dataStyle->getFont()->setSize(12);
        $dataStyle->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_LEFT);

        return $spreadsheet;
    }

    /**
     * Gera o layout da planilha de acordo com o tipo da solicitação.
     *
     * @param array $solicitacoes Array com os dados das solicitações.
     * @param int $nrTipoSolicitacao int com o tipo da solicitação.
     * @return Spreadsheet O objeto Spreadsheet com os dados formatados.
     */
    public function buildSolicitacoesEspelhoXls($nrorg, $solicitacoes, $nrTipoSolicitacao): Spreadsheet
    {
        $labelFieldPessoa = MinhaHistoriaService::$labelFieldPessoa;
        $TIPO_SOLICITACAO_CAMPOS = MinhaHistoriaService::TIPO_SOLICITACAO_CAMPOS;
        $spreadsheet = new Spreadsheet();
        $sheet = $spreadsheet->getActiveSheet();
    
        // Definir cabeçalhos fixos
        $headers = [
            'Nº Vínculo',
            'Vínculo',
            'Estrutura Legal',
            'Estrutura Gerencial',
            'Nº Solicitação',
            'Data de Solicitação',
            'Tipo Solicitação',
            'Aprovação',
            'Fase Atual',
            'Solicitante'
        ];
    
        // Adicionar os cabeçalhos fixos
        $col = 'A';
        foreach ($headers as $header) {
            $sheet->setCellValue($col . '1', $header);
            $sheet->getColumnDimension($col)->setAutoSize(true);
            $col++;
        }
    
        // Incluir colunas dinâmicas baseadas no nrTipoSolicitacao (somente os labels)
        $dynamicColumns = [];
        $tipoSolicitacaoExiste = isset($TIPO_SOLICITACAO_CAMPOS[$nrTipoSolicitacao]);

        if ($tipoSolicitacaoExiste) {
            $camposFiltrados = $this->filterSolicitacaoCampos($nrorg, $TIPO_SOLICITACAO_CAMPOS[$nrTipoSolicitacao], $nrTipoSolicitacao);

            foreach ($camposFiltrados as $campoDinamico) {
                if (!empty($labelFieldPessoa[$campoDinamico])) {
                    $sheet->setCellValue($col . '1', $labelFieldPessoa[$campoDinamico]);
                    $sheet->getColumnDimension($col)->setAutoSize(true);
                    $dynamicColumns[$campoDinamico] = $col; // Mapeia o campo dinâmico para a coluna
                    $col++;
                }
            }
        }

        // Calculando a última coluna corretamente
        $lastColumnIndex = $this->columnToIndex($col);
        $lastColumn = Coordinate::stringFromColumnIndex($lastColumnIndex);

        // Aplicar o estilo ao cabeçalho
        $headerStyle = $sheet->getStyle('A1:' . $lastColumn . '1');
        $headerStyle->getFont()->setSize(12)->setBold(true);
        $headerStyle->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER);
    
        // Preencher os dados
        $row = 2; // Começa na segunda linha (depois dos cabeçalhos)
        foreach ($solicitacoes as $solicitacao) {
            $col = 'A';
            $sheet->setCellValue($col++ . $row, $solicitacao['NRVINCULOM']        ?? '');
            $sheet->setCellValue($col++ . $row, $solicitacao['NMVINCULOM']        ?? '');
            $sheet->setCellValue($col++ . $row, $solicitacao['NMESTRUTLEGAL']     ?? '');
            $sheet->setCellValue($col++ . $row, $solicitacao['NMESTRUTGEREN']     ?? '');
            $sheet->setCellValue($col++ . $row, $solicitacao['NRSOLICITACAO']     ?? '');
            $sheet->setCellValue($col++ . $row, $solicitacao['DTSOLICITACAO']     ?? '');
            $sheet->setCellValue($col++ . $row, $solicitacao['NMTIPOSOLICITACAO'] ?? '');
            $sheet->setCellValue($col++ . $row, $solicitacao['DSAPROVACAOSEC']    ?? '');
            $sheet->setCellValue($col++ . $row, $solicitacao['NMFASEAPROVSOLI']   ?? '');
            $sheet->setCellValue($col++ . $row, $solicitacao['NMOPERADOR']        ?? '');
    
            // Preencher colunas dinâmicas
            if (isset($solicitacao['ITENS']) && is_array($solicitacao['ITENS'])) {
                foreach ($solicitacao['ITENS'] as $item) {
                    $chave = $item['DSCHAVEITEMSOLI'] ?? null;
                    $valor = $item['DSVALORITEMSOLI'] ?? null;
    
                    // Se a chave existir no mapeamento de colunas dinâmicas, preenche o valor na coluna correspondente
                    if ($chave && isset($dynamicColumns[$chave])) {
                        $sheet->setCellValue($dynamicColumns[$chave] . $row, $valor);
                    }
                }
            }
    
            $row++;
        }

        // Aplicar estilo aos dados
        $dataStyle = $sheet->getStyle('A2:' . $lastColumn . ($row - 1));
        $dataStyle->getFont()->setSize(12);
        $dataStyle->getAlignment()->setHorizontal(\PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_LEFT);

        // Aplicar cor azul claro às colunas dinâmicas
        $headerStyle = [
            'fill' => [
                'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                'startColor' => ['rgb' => 'ADD8E6'], // Azul claro
            ],
        ];

        foreach ($dynamicColumns as $col) {
            $sheet->getStyle($col . '1')->applyFromArray($headerStyle);
        }
    
        return $spreadsheet;
    }

    // Função para converter a coluna (ex: 'A', 'Z', 'AA', etc.) para o índice
    function columnToIndex($column) {
        $index = 0;
        $length = strlen($column);
        for ($i = 0; $i < $length; $i++) {
            $index = $index * 26 + (ord($column[$i]) - ord('A') + 1);
        }
        return $index;
    }


    // Aplicando filtro específico para a organização 1410 - Masan  
    // Removendo itens que não são exibidos na solicitação dessa organização
    private function filterSolicitacaoCampos($nrorg, $campos, $nrTipoSolicitacao): array 
    {
        $camposParaRemover = [
            ProcessamentoSolicitacaoFactory::TP_SOLI_SUBSTITUICAO => ['NMPROCESSO', 'NRTEMPLATE', 'NRTIPOPROCESSO', 'DSMOTIVO', 'LOCALIDADE', 'DSDESCRICAO', 'DTSOLICITA', 'DTLIMSOLICITA', 'NRGRAUINSTR'],
            ProcessamentoSolicitacaoFactory::TP_SOLI_AUMENTO_QUADRO => ['NMPROCESSO', 'NRTEMPLATE', 'NRTIPOPROCESSO', 'DSMOTIVO', 'LOCALIDADE', 'DSDESCRICAO', 'DTSOLICITA', 'DTLIMSOLICITA']
        ];
        
        if ($nrorg === 1410 && isset($camposParaRemover[$nrTipoSolicitacao])) {
            return array_diff($campos, $camposParaRemover[$nrTipoSolicitacao]);
        }
        
        return $campos;
    }

    
}
