<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Repositories;
use Util\Environment;
use Util\Messages;
use Ems\Util\ArrayUtil;
use Ems\Services\{ OperadorService };

use Ems\Repository\ControleAcessoRepository;
use Geral\Util\NovoCodigo;

// MENSAGENS
define('messagesControleAcessoService', [
    'ERRO_PERFIL_NAO_AUTORIZADO'     => 'Operador não possui perfil de acesso para alterar o controle de acesso.',
]);

class ControleAcessoService {

    /** @var ControleAcessoRepository */
    protected $controleAcessoRepository;

    /** @var OperadorService */
    protected $operadorService;

    /** @var Environment */
    protected $environment;

    public function __construct(
        EntityManager $entityManager,
        Environment $environment,
        ControleAcessoRepository $controleAcessoRepository,
        OperadorService $operadorService
    ) {
        $this->entityManager              = $entityManager;
        $this->environment                = $environment;
        $this->controleAcessoRepository   = $controleAcessoRepository;
        $this->operadorService            = $operadorService;
    }

    public function getPerfisAcessoControle(): array {
        $nrOrg = $this->environment->getNrOrg();
        $nrOrgPadrao = $this->environment->getNrOrgPadrao();
        
        $params = array(
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
        );
        
        return $this->controleAcessoRepository->getPerfisAcessoControle($params);
    }

    public function getPermissoesAcessoPerfil(
        $nrPerfilAcesso,
        $projeto,
        $cdOperador
        ) {
        $nrOrg      = (int) $this->environment->getNrOrg();
        
        $this->validaOperadorIsDP($cdOperador);

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRPERFILACESSO'  => $nrPerfilAcesso,
            'P_PROJETO'         => $projeto
        ];

        $permissoes = $this->controleAcessoRepository->getPermissoesAcessoPerfil($params);

        // Transformar 'S' em true e 'N' em false
        foreach ($permissoes as &$permissao) {
            $permissao['IDVISUALIZAR'] = $permissao['IDVISUALIZAR'] === 'S';
            $permissao['IDCRIAR'] = $permissao['IDCRIAR'] === 'S';
            $permissao['IDATUALIZAR'] = $permissao['IDATUALIZAR'] === 'S';
            $permissao['IDDELETAR'] = $permissao['IDDELETAR'] === 'S';
        }

        return $permissoes;
    }

    public function atualizarListaPermissoesAcesso($nrPerfilAcesso, $permissoes, $cdOperador) {
        $success = true;

        $this->validaOperadorIsDP($cdOperador);
    
        foreach ($permissoes as $permissao) {
            $nrControleModulo = $permissao['NRCONTROLEMODULO'] ?? null;
            $visualizar       = $permissao['IDVISUALIZAR']     ?? null;
            $criar            = $permissao['IDCRIAR']          ?? null;
            $atualizar        = $permissao['IDATUALIZAR']      ?? null;
            $deletar          = $permissao['IDDELETAR']        ?? null;
    
            $this->processarPermissaoAcesso($nrPerfilAcesso, $nrControleModulo, $visualizar, $criar, $atualizar, $deletar, $cdOperador);
        }
    
        return $success;
    }
    
    public function processarPermissaoAcesso(
        $nrPerfilAcesso,
        $nrControleModulo,
        $visualizar,
        $criar,
        $atualizar,
        $deletar,
        $cdOperador
    ) {

        $permissaoAcesso = $this->getPermissaoAcessoByPerfilModulo($nrPerfilAcesso, $nrControleModulo);

        $nrControlePermissao = $permissaoAcesso['NRCONTROLEPERMISSAO'] ?? null;

        if ($nrControlePermissao !== null) {

            $this->atualizarPermissaoAcessoById(
            $nrControlePermissao,
            $nrPerfilAcesso,
            $nrControleModulo,
            $visualizar,
            $criar,
            $atualizar,
            $deletar,
            $cdOperador);

        } else {

            $this->criarPermissaoAcesso(
            $nrPerfilAcesso,
            $nrControleModulo,
            $visualizar,
            $criar,
            $atualizar,
            $deletar,
            $cdOperador);

        }
    }


    private function criarPermissaoAcesso(
        $nrPerfilAcesso,
        $nrControleModulo,
        $visualizar,
        $criar,
        $atualizar,
        $deletar,
        $cdOperador
    ) {
        $nrOrg      = (int) $this->environment->getNrOrg();
        $nrControlePermissao = NovoCodigo::geraCodigo($nrOrg, 'GPE_CONTROLE_PERMISSAO', 1, 12, true);

        $params = [
            'P_NRORG'                 => $nrOrg,
            'P_NRCONTROLEPERMISSAO'   => $nrControlePermissao,
            'P_NRCONTROLEMODULO'      => $nrControleModulo,
            'P_NRPERFILACESSO'        => $nrPerfilAcesso,
            'P_IDVISUALIZAR'          => $visualizar ? 'S' : 'N',
            'P_IDCRIAR'               => $criar      ? 'S' : 'N',
            'P_IDATUALIZAR'           => $atualizar  ? 'S' : 'N',
            'P_IDDELETAR'             => $deletar    ? 'S' : 'N',
            'P_CDOPERADOR'            => $cdOperador
        ];


        $this->controleAcessoRepository->criarPermissaoAcesso($params);
    }

    public function atualizarPermissaoAcessoById(
        $nrControlePermissao,
        $nrPerfilAcesso,
        $nrControleModulo,
        $visualizar,
        $criar,
        $atualizar,
        $deletar,
        $cdOperador
    ) {
        $nrOrg      = (int) $this->environment->getNrOrg();

        $params = [
            'P_NRCONTROLEPERMISSAO'   => $nrControlePermissao,
            'P_NRORG'                 => $nrOrg,
            'P_NRCONTROLEMODULO'      => $nrControleModulo,
            'P_NRPERFILACESSO'        => $nrPerfilAcesso,
            'P_IDVISUALIZAR'          => $visualizar ? 'S' : 'N',
            'P_IDCRIAR'               => $criar      ? 'S' : 'N',
            'P_IDATUALIZAR'           => $atualizar  ? 'S' : 'N',
            'P_IDDELETAR'             => $deletar    ? 'S' : 'N',
            'P_CDOPERADOR'            => $cdOperador
        ];

        $this->controleAcessoRepository->atualizarPermissaoAcessoById($params);
    }

    public function atualizarStatusControleAcesso($status, $cdOperador) {

        $nrOrg = (int) $this->environment->getNrOrg();

        $this->validaOperadorIsDP($cdOperador);

        $params = [
            'P_IDATIVACONTROLEACESSO' => $status,
            'P_NRORG'                 => $nrOrg,
            'P_CDOPERADOR'            => $cdOperador
        ];

        $this->controleAcessoRepository->atualizarStatusControleAcesso($params);

        return true;
    }


    private function validaOperadorIsDP($cdOperador) {
        $cdOperador = (string) $cdOperador;

        $operadorIsDp = $this->operadorService->operadorIsDP($cdOperador);

        if (!$operadorIsDp) {
            throw new \Exception(constant('messagesControleAcessoService')['ERRO_PERFIL_NAO_AUTORIZADO']);
        }

    }

    private function getPermissaoAcessoByPerfilModulo($nrPerfilAcesso, $nrControleModulo) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $permissao = $this->controleAcessoRepository->getPermissaoAcessoByPerfilModulo($nrOrg, $nrPerfilAcesso, $nrControleModulo);

        return $permissao;
    }
    


}